@@ -1,7 +1,10 @@
 #include "gcin.h"
 #include "config.h"
+#include "gcin-version.h"
 #if UNIX
 #include <signal.h>
+#endif
+#if GCIN_i18n_message
 #include <libintl.h>
 #endif
 
@@ -66,20 +69,20 @@ char *lc;
 
 static XIMStyle Styles[] = {
 #if 1
-        XIMPreeditCallbacks|XIMStatusCallbacks,		//OnTheSpot
-        XIMPreeditCallbacks|XIMStatusArea,		//OnTheSpot
-        XIMPreeditCallbacks|XIMStatusNothing,		//OnTheSpot
+        XIMPreeditCallbacks|XIMStatusCallbacks,         //OnTheSpot
+        XIMPreeditCallbacks|XIMStatusArea,              //OnTheSpot
+        XIMPreeditCallbacks|XIMStatusNothing,           //OnTheSpot
 #endif
-        XIMPreeditPosition|XIMStatusArea,		//OverTheSpot
-        XIMPreeditPosition|XIMStatusNothing,		//OverTheSpot
-        XIMPreeditPosition|XIMStatusNone,		//OverTheSpot
+        XIMPreeditPosition|XIMStatusArea,               //OverTheSpot
+        XIMPreeditPosition|XIMStatusNothing,            //OverTheSpot
+        XIMPreeditPosition|XIMStatusNone,               //OverTheSpot
 #if 1
-        XIMPreeditArea|XIMStatusArea,			//OffTheSpot
-        XIMPreeditArea|XIMStatusNothing,		//OffTheSpot
-        XIMPreeditArea|XIMStatusNone,			//OffTheSpot
+        XIMPreeditArea|XIMStatusArea,                   //OffTheSpot
+        XIMPreeditArea|XIMStatusNothing,                //OffTheSpot
+        XIMPreeditArea|XIMStatusNone,                   //OffTheSpot
 #endif
-        XIMPreeditNothing|XIMStatusNothing,		//Root
-        XIMPreeditNothing|XIMStatusNone,		//Root
+        XIMPreeditNothing|XIMStatusNothing,             //Root
+        XIMPreeditNothing|XIMStatusNone,                //Root
 };
 static XIMStyles im_styles;
 
@@ -118,10 +121,10 @@ int MyTriggerNotifyHandler(IMTriggerNotifyStruct *call_data)
             ) {
             toggle_im_enabled();
         }
-	return True;
+        return True;
     } else {
-	/* never happens */
-	return False;
+        /* never happens */
+        return False;
     }
 }
 
@@ -150,18 +153,18 @@ int gcin_ProtoHandler(XIMS ims, IMProtocol *call_data)
       if(pimopen->connect_id > MAX_CONNECT - 1)
         return True;
 
-#if DEBUG
+#if DEBUG && 0
     dbg("open lang %s  connectid:%d\n", pimopen->lang.name, pimopen->connect_id);
 #endif
       return True;
     }
   case XIM_CLOSE:
-#if DEBUG
+#if DEBUG && 0
     dbg("XIM_CLOSE\n");
 #endif
     return True;
   case XIM_CREATE_IC:
-#if DEBUG
+#if DEBUG && 0
      dbg("CREATE_IC\n");
 #endif
      CreateIC((IMChangeICStruct *)call_data);
@@ -169,62 +172,62 @@ int gcin_ProtoHandler(XIMS ims, IMProtocol *call_data)
   case XIM_DESTROY_IC:
      {
        IMChangeICStruct *pimcha=(IMChangeICStruct *)call_data;
-#if DEBUG
+#if DEBUG && 0
        dbg("DESTROY_IC %d\n", pimcha->icid);
 #endif
        DeleteIC(pimcha->icid);
      }
      return True;
   case XIM_SET_IC_VALUES:
-#if DEBUG
+#if DEBUG && 0
      dbg("SET_IC\n");
 #endif
      SetIC((IMChangeICStruct *)call_data);
      return True;
   case XIM_GET_IC_VALUES:
-#if DEBUG
+#if DEBUG && 0
      dbg("GET_IC\n");
 #endif
      GetIC((IMChangeICStruct *)call_data);
      return True;
   case XIM_FORWARD_EVENT:
-#if DEBUG
+#if DEBUG && 0
      dbg("XIM_FORWARD_EVENT\n");
 #endif
      return xim_ForwardEventHandler((IMForwardEventStruct *)call_data);
   case XIM_SET_IC_FOCUS:
-#if DEBUG
+#if DEBUG && 0
      dbg("XIM_SET_IC_FOCUS\n");
 #endif
      return xim_gcin_FocusIn((IMChangeFocusStruct *)call_data);
   case XIM_UNSET_IC_FOCUS:
-#if DEBUG
+#if DEBUG && 0
      dbg("XIM_UNSET_IC_FOCUS\n");
 #endif
      return xim_gcin_FocusOut((IMChangeFocusStruct *)call_data);
   case XIM_RESET_IC:
-#if DEBUG
+#if DEBUG && 0
      dbg("XIM_UNSET_IC_FOCUS\n");
 #endif
      return True;
   case XIM_TRIGGER_NOTIFY:
-#if DEBUG
+#if DEBUG && 0
      dbg("XIM_TRIGGER_NOTIFY\n");
 #endif
      MyTriggerNotifyHandler((IMTriggerNotifyStruct *)call_data);
      return True;
   case XIM_PREEDIT_START_REPLY:
-#if DEBUG
+#if DEBUG && 0
      dbg("XIM_PREEDIT_START_REPLY\n");
 #endif
      return True;
   case XIM_PREEDIT_CARET_REPLY:
-#if DEBUG
+#if DEBUG && 0
      dbg("XIM_PREEDIT_CARET_REPLY\n");
 #endif
      return True;
   case XIM_STR_CONVERSION_REPLY:
-#if DEBUG
+#if DEBUG && 0
      dbg("XIM_STR_CONVERSION_REPLY\n");
 #endif
      return True;
@@ -277,10 +280,12 @@ void open_xim()
 void load_tsin_db();
 void load_tsin_conf(), load_setttings(), load_tab_pho_file();
 
-void disp_hide_tsin_status_row(), gcb_main(), update_win_kbm();
+void disp_hide_tsin_status_row(), gcb_main(), update_win_kbm_inited();
 void change_tsin_line_color(), change_win0_style(), change_tsin_color();
 void change_win_gtab_style();
 void update_item_active_all();
+void destroy_inmd_menu();
+void load_gtab_list(gboolean);
 
 static void reload_data()
 {
@@ -292,8 +297,10 @@ static void reload_data()
 //  change_win_pho_style();
   load_tab_pho_file();
   change_tsin_color();
-  if (win_kbm_inited)
-    update_win_kbm();
+  update_win_kbm_inited();
+
+  destroy_inmd_menu();
+  load_gtab_list(TRUE);
 
   update_item_active_all();
 #if USE_GCB
@@ -306,6 +313,7 @@ void change_gtab_font_size();
 void change_pho_font_size();
 void change_win_sym_font_size();
 void change_win_gtab_style();
+extern int win_kbm_on;
 
 static void change_font_size()
 {
@@ -316,6 +324,7 @@ static void change_font_size()
   change_win_sym_font_size();
   change_win0_style();
   change_win_gtab_style();
+  update_win_kbm_inited();
 //  change_win_pho_style();
 }
 
@@ -352,6 +361,7 @@ void kbm_toggle()
 
 
 void reload_tsin_db();
+void do_exit();
 
 void message_cb(char *message)
 {
@@ -374,8 +384,10 @@ void message_cb(char *message)
 #endif
    if (!strcmp(message, RELOAD_TSIN_DB)) {
      reload_tsin_db();
-   }
-   else
+   } else
+   if (!strcmp(message, GCIN_EXIT_MESSAGE)) {
+     do_exit();
+   } else
      reload_data();
 }
 
@@ -449,9 +461,14 @@ void sig_do_exit(int sig)
 }
 
 char *get_gcin_xim_name();
-void load_phrase(), init_TableDir(),  load_gtab_list();
-void init_im_serv(), init_tray(), exec_setup_scripts();
-void init_gcin_im_serv(Window win), gcb_main(), init_tray_win32();
+void load_phrase(), init_TableDir();
+void init_tray(), exec_setup_scripts();
+#if UNIX
+void init_gcin_im_serv(Window win);
+#else
+void init_gcin_im_serv();
+#endif
+void gcb_main(), init_tray_win32();
 
 #if WIN32
 void init_gcin_program_files();
@@ -491,32 +508,46 @@ gboolean delayed_start_cb(gpointer data)
   return FALSE;
 }
 
+void get_dpy_xyl()
+{
+	dpy_xl = gdk_screen_width(), dpy_yl = gdk_screen_height();
+}
+
+void screen_size_changed(GdkScreen *screen, gpointer user_data)
+{
+	get_dpy_xyl();
+}
 
 int main(int argc, char **argv)
 {
+#if UNIX
+  signal(SIGCHLD, SIG_IGN);
+  signal(SIGPIPE, SIG_IGN);
+
+  if (getenv("GCIN_DAEMON")) {
+    daemon(1,1);
+#if FREEBSD
+    setpgid(0, getpid());
+#else
+    setpgrp();
+#endif
+  }
+#endif
 
 //putenv("GDK_NATIVE_WINDOWS=1");
 #if WIN32
 #if 1
-	typedef BOOL (WINAPI* pImmDisableIME)(DWORD);
-	pImmDisableIME pd;
-	HMODULE imm32=LoadLibraryA("imm32");
-//	printf("****** imm32 %x\n", imm32);
-	if (imm32 && (pd=(pImmDisableIME)GetProcAddress(imm32, "ImmDisableIME"))) {
-//		puts("imm loaded");
-		(*pd)(0);
-	}
+        typedef BOOL (WINAPI* pImmDisableIME)(DWORD);
+        pImmDisableIME pd;
+        HMODULE imm32=LoadLibraryA("imm32");
+//      printf("****** imm32 %x\n", imm32);
+        if (imm32 && (pd=(pImmDisableIME)GetProcAddress(imm32, "ImmDisableIME"))) {
+//              puts("imm loaded");
+                (*pd)(0);
+        }
 #endif
   init_gcin_program_files();
-
-  init_gcin_im_serv(NULL);
-#endif
-
-
-#if GCIN_i18n_message
-  gtk_set_locale();
-  bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
-  textdomain(GETTEXT_PACKAGE);
+  init_gcin_im_serv();
 #endif
 
 #if USE_XIM
@@ -535,10 +566,6 @@ int main(int argc, char **argv)
   if (!lc_ctype)
     lc_ctype = "zh_TW.Big5";
 
-
-  signal(SIGCHLD, SIG_IGN);
-  signal(SIGPIPE, SIG_IGN);
-
   char *t = strchr(lc_ctype, '.');
   if (t) {
     int len = t - lc_ctype;
@@ -568,17 +595,23 @@ int main(int argc, char **argv)
 
   init_TableDir();
   load_setttings();
-  load_gtab_list();
+  load_gtab_list(TRUE);
 
   gtk_init (&argc, &argv);
 
+#if GCIN_i18n_message
+  bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+  textdomain(GETTEXT_PACKAGE);
+#endif
+
   dbg("after gtk_init\n");
 
 #if UNIX
   dpy = GDK_DISPLAY();
   root=DefaultRootWindow(dpy);
 #endif
-  dpy_xl = gdk_screen_width(), dpy_yl = gdk_screen_height();
+  get_dpy_xyl();
+  g_signal_connect(gdk_screen_get_default(),"size-changed", G_CALLBACK(screen_size_changed), NULL);
 
   dbg("display width:%d height:%d\n", dpy_xl, dpy_yl);
 
