@@ -828,6 +828,7 @@ int show_buf_select()
   ggg.pg_idx = 0;
 
   gtab_disp_sel();
+  hide_gtab_pre_sel();
 
   return 1;
 }
@@ -867,6 +868,10 @@ int gtab_get_preedit(char *str, GCIN_PREEDIT_ATTR attr[], int *pcursor, int *sub
 
 #if WIN32
   *sub_comp_len = ggg.ci > 0;
+#if 1
+  if (ggg.gbufN && !gcin_edit_display_ap_only())
+	*sub_comp_len|=4;
+#endif
 #endif
   gboolean ap_only = gcin_edit_display_ap_only();
 
@@ -1072,8 +1077,10 @@ void gtab_scan_pre_select(gboolean b_incr)
 
 //  dbg("max_len:%d  max_selN:%d\n", max_len, max_selN);
 
-  if (max_len < 0 || max_selN >= strlen(cur_inmd->selkey) * 2)
+  if (max_len < 0 || max_selN >= strlen(cur_inmd->selkey) * 2) {
+    tss.pre_selN = 0;
     return;
+  }
 
   gtab_pre_select_phrase_len = max_len;
 
@@ -1130,6 +1137,9 @@ gboolean gtab_pre_select_idx(int c)
   gbuf[ggg.gbufN-1].flag |= FLAG_CHPHO_PHRASE_TAIL;
 
   hide_gtab_pre_sel();
+  if (gcin_edit_display==GCIN_EDIT_DISPLAY_ON_THE_SPOT)
+    hide_win_gtab();
+
   return TRUE;
 }
 
