@@ -221,7 +221,7 @@ static void putbuf(int len)
 #endif
 
   for(idx=i=0;i<len;i++) {
-    int len = strlen(tss.chpho[i].ch);
+    int len = utf8_sz(tss.chpho[i].ch);
 
     if (tss.chpho[i].pho && len > 1) {
       int pho_idx = ch_key_to_ch_pho_idx(tss.chpho[i].pho, tss.chpho[i].ch);
@@ -1010,7 +1010,7 @@ void ch_pho_cpy(CHPHO *pchpho, char *utf8, phokey_t *phos, int len)
   int i;
 
   for(i=0; i < len; i++) {
-    int len = u8cpy(pchpho[i].cha, utf8);
+    int len = utf8cpy(pchpho[i].cha, utf8);
     utf8+=len;
     pchpho[i].pho = phos[i];
 	pchpho[i].flag &= ~FLAG_CHPHO_PHO_PHRASE;
@@ -1150,7 +1150,6 @@ gboolean add_to_tsin_buf_phsta(char *str, phokey_t *pho, int len)
 }
 
 
-#if USE_TSIN
 void add_to_tsin_buf_str(char *str)
 {
   char *pp = str;
@@ -1167,12 +1166,12 @@ void add_to_tsin_buf_str(char *str)
       break;
   }
 
+  dbg("add_to_tsin_buf_str %s %d\n",str, N);
 
   phokey_t pho[MAX_PHRASE_LEN];
   bzero(pho, sizeof(pho));
   add_to_tsin_buf(str, pho, N);
 }
-#endif
 
 int tsin_pho_sel(int c);
 
@@ -1238,13 +1237,9 @@ static gboolean pre_sel_handler(KeySym xkey)
   return tsin_sele_by_idx(c);
 }
 
-
-static gboolean pre_punctuation(KeySym xkey)
+static gboolean pre_punctuation_sub(KeySym xkey, char shift_punc[], unich_t *chars[])
 {
   char *p;
-  static char shift_punc[]="<>?:\"{}!";
-  static unich_t *chars[]={_L("，"),_L("。"),_L("？"),_L("："),_L("；"),_L("『"),_L("』"),_L("！")};
-
   if ((p=strchr(shift_punc, xkey))) {
     int c = p - shift_punc;
     phokey_t key=0;
@@ -1259,28 +1254,18 @@ static gboolean pre_punctuation(KeySym xkey)
 }
 
 
-static char hsu_punc[]=",./;";
+static gboolean pre_punctuation(KeySym xkey)
+{
+  static char shift_punc[]="<>?:\"{}!";
+  static unich_t *chars[]={_L("，"),_L("。"),_L("？"),_L("："),_L("；"),_L("『"),_L("』"),_L("！")};
+  return pre_punctuation_sub(xkey, shift_punc, chars);
+}
+
+static char hsu_punc[]=",./;'";
 static gboolean pre_punctuation_hsu(KeySym xkey)
 {
-
-  static char *chars[]={
-#if UNIX
-	  "，","。","？","；"
-#else
-"\xef\xbc\x8c",
-"\xe3\x80\x82","\xef\xbc\x9f","\xef\xbc\x9b"
-#endif
-  };
-  char *p;
-
-  if ((p=strchr(hsu_punc, xkey))) {
-    int c = p - hsu_punc;
-    phokey_t key=0;
-
-    return add_to_tsin_buf(chars[c], &key, 1);
-  }
-
-  return 0;
+  static unich_t *chars[]={_L("，"),_L("。"),_L("？"),_L("；"),_L("、")};
+  return pre_punctuation_sub(xkey, hsu_punc, chars);
 }
 
 
@@ -1800,15 +1785,10 @@ change_char:
 		 return win_sym_page_down();
 	   }
      case '\'':  // single quote
-       if (phkbm.phokbm[xkey][0].num)
+       if (phkbm.phokbm[xkey][0].num && !pin_juyin)
          goto other_keys;
        else {
-         phokey_t key = 0;
-#if UNIX
-         return add_to_tsin_buf("、", &key, 1);
-#else
-		 return add_to_tsin_buf("\xe3\x80\x81", &key, 1);
-#endif
+	 return pre_punctuation_hsu(xkey);
        }
      case 'q':
      case 'Q':
@@ -1890,15 +1870,18 @@ other_keys:
        if (xkey >= XK_KP_0 && xkey<=XK_KP_9)
          xkey_lcase = xkey - XK_KP_0 + '0';
 
+	   gboolean use_pre_sel;
+	   use_pre_sel = tss.pre_selN && !tss.sel_pho && xkey < 127 && !phkbm.phokbm[xkey][0].num;
+
        char *pp;
-       if ((pp=strchr(pho_selkey,xkey_lcase)) && (tss.sel_pho || tss.ctrl_pre_sel)) {
+	   if ((pp=strchr(pho_selkey,xkey_lcase)) && (tss.sel_pho || tss.ctrl_pre_sel || use_pre_sel)) {
          int c=pp-pho_selkey;
 
 		 if (tss.sel_pho) {
            if (tsin_pho_sel(c))
              return 1;
 		 } else
-		 if (tss.ctrl_pre_sel) {
+		 if (tss.ctrl_pre_sel || use_pre_sel) {
   		     tss.ctrl_pre_sel = FALSE;
 			 if (tsin_sele_by_idx(c))
 				 return TRUE;
@@ -2233,6 +2216,10 @@ fin:
   *sub_comp_len = !typ_pho_empty();
   if (gwin1 && GTK_WIDGET_VISIBLE(gwin1))
     *sub_comp_len|=2;
+#if 1
+  if (tss.c_len && !gcin_edit_display_ap_only())
+	*sub_comp_len|=4;
+#endif
 #endif
 
   return attrN;
